<?php
// +----------------------------------------------------------------------
// | [KyPHP System] Copyright (c) 2020 http://www.kuryun.com/
// +----------------------------------------------------------------------
// | [KyPHP] 并不是自由软件,你可免费使用,未经许可不能去掉KyPHP相关版权
// +----------------------------------------------------------------------
// | Author: fudaoji <fdj@kuryun.cn>
// +----------------------------------------------------------------------

/**
 * Created by PhpStorm.
 * Script Name: App.php
 * Create: 2020/5/17 下午5:08
 * Description: 应用中心
 * Author: fudaoji<fdj@kuryun.cn>
 */

namespace app\admin\controller;

use app\common\model\AdminStore;
use think\facade\Log;

class App extends Base
{
    /**
     * @var \app\common\model\Addons
     */
    private $addonM;
    /**
     * @var \app\common\model\AddonsInfo
     */
    private $addonInfoM;

    public function initialize()
    {
        parent::initialize(); // TODO: Change the autogenerated stub
        $this->addonM = model('addons');
        $this->addonInfoM = model('addonsInfo');
    }

    /**
     * 参数配置
     */
    public function config()
    {
        $id = input('id', 0, 'intval');
        $addon = $this->addonM->getOne($id,true);
        if (request()->isPost()) {
            $post_data = input('post.');
            $data['id'] = $post_data['id'];
            $configs = [
                'request_domain' => explode(';', $post_data['request_domain']),
                'socket_domain' => explode(';', $post_data['socket_domain']),
                'upload_domain' => explode(';', $post_data['upload_domain']),
                'download_domain' => explode(';', $post_data['download_domain']),
                'udp_domain' => explode(';', $post_data['udp_domain']),
                'tcp_domain' => explode(';', $post_data['tcp_domain']),
                'webview_domain' => explode(';', $post_data['webview_domain']),
                'ext_json' => [],
                'contact_phone' => $post_data['contact_phone'],
                'notice_method' => $post_data['notice_method'],
                'setting_list' => [],
            ];
            $ext_json_arr = preg_split('/[\r\n]+/', trim($post_data['ext_json'], "\r\n"));
            foreach ($ext_json_arr as $item){
                list($k, $v) = explode('|', $item);
                $configs['ext_json'][$k] = $v;
            }
            $setting_arr = preg_split('/[\r\n]+/', trim($post_data['setting_list'], "\r\n"));
            foreach ($setting_arr as $item){
                list($k, $v) = explode('|', $item);
                $configs['setting_list'][$k] = $v;
            }
            $data['config'] = json_encode($configs, JSON_UNESCAPED_UNICODE);

            $this->addonM->updateOne($data);
            $this->success('配置成功');
        } else {
            $config = json_decode($addon['config'], true);
            if(empty($config['request_domain'])){
                $http_host = request()->server()['HTTP_HOST'];
                $config = [
                    'request_domain' => ['https://' . $http_host],
                    'socket_domain' => ['wss://' . $http_host],
                    'upload_domain' => ['https://' . $http_host],
                    'download_domain' => ['https://' . $http_host],
                    'udp_domain' => [],
                    'tcp_domain' => [],
                    'webview_domain' => ['https://' . $http_host],
                    'ext_json' => '',
                    'contact_phone' => $this->adminInfo['mobile']
                ];
            }else{
                $ext_json_arr = [];
                foreach ($config['ext_json'] as $k => $v){
                    $ext_json_arr[] .= $k . '|' . $v;
                }
                $config['ext_json'] = implode("\r\n", $ext_json_arr);
                if(!empty($config['setting_list'])){
                    $setting_arr = [];
                    foreach ($config['setting_list'] as $k => $v){
                        $setting_arr[] .= $k . '|' . $v;
                    }
                    $config['setting_list'] = implode("\r\n", $setting_arr);
                }
            }
            $config['request_domain'] = implode(';', $config['request_domain']);
            $config['socket_domain'] = implode(';', $config['socket_domain']);
            $config['upload_domain'] = implode(';', $config['upload_domain']);
            $config['download_domain'] = implode(';', $config['download_domain']);
            $config['udp_domain'] = implode(';', $config['udp_domain']);
            $config['tcp_domain'] = implode(';', $config['tcp_domain']);
            $config['webview_domain'] = implode(';', $config['webview_domain']);

            $builder = new FormBuilder();
            $builder->setPostUrl(url('config'))
                ->setTip('多个域名用;隔开')
                ->addFormItem('id', 'hidden', 'id', 'id')
                ->addFormItem('domains', 'legend', '服务器域名', '服务器域名')
                ->addFormItem('request_domain', 'textarea', 'request合法域名', '多个域名使用;隔开')
                ->addFormItem('socket_domain', 'textarea', 'socket合法域名', '多个域名使用;隔开')
                ->addFormItem('upload_domain', 'textarea', 'uploadFile合法域名', '多个域名使用;隔开')
                ->addFormItem('download_domain', 'textarea', 'downloadFile合法域名', '多个域名使用;隔开')
                ->addFormItem('udp_domain', 'textarea', 'udp合法域名', '多个域名使用;隔开')
                ->addFormItem('tcp_domain', 'textarea', 'tcp合法域名', '多个域名使用;隔开')
                ->addFormItem('webview', 'legend', '业务域名', '业务域名')
                ->addFormItem('webview_domain', 'textarea', '业务域名', '多个域名使用;隔开')
                ->addFormItem('extJson', 'legend', 'extJson参数', 'extJson参数')
                ->addFormItem('ext_json', 'textarea', 'extJson', '一行一对配置，例如：
restUrl|http://demo.com
appKey|123456')
                ->addFormItem('privacy', 'legend', '用户隐私设置', '用户隐私保护指引参数设置')
                ->addFormItem('contact_phone', 'number', '开发者手机号', '请填写真实手机号',[], 'required')
                ->addFormItem('notice_method', 'text', '通知方式', '指的是当开发者收集信息有变动时，通过该方式通知用户。这里服务商需要按照实际情况填写，例如通过弹窗或者公告或者其他方式',[], 'required')
                ->addFormItem('setting_list', 'textarea', '要收集的信息', '一行一对配置，例如：
UserInfo|个人中心显示个人信息
Location|提供最近的服务');

            $config['id'] = $addon['id'];
            $builder->setFormData($config);
            return $builder->show();
        }
    }

    /**
     * 为小程序应用设置模版
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function setTemplate(){
        if(request()->isPost()){
            $post_data = input('post.');
            $mini_template = model('addonsTemplate')->getOneByMap(['addon' => $post_data['addon']], true, true);
            if($mini_template){
                $res = model('addonsTemplate')->updateOne(['id' => $mini_template['id'], 'template_id' => $post_data['template_id']]);
            }else{
                $res = model('addonsTemplate')->addOne(['addon' => $post_data['addon'], 'template_id' => $post_data['template_id']]);
            }
            if($res){
                $this->success('设置成功', url('index', ['type' => 'onsale']));
            }else{
                $this->error('系统出错，请刷新重试','', ['token' => request()->token()]);
            }
        }
        if(! $data = $this->addonM->getOne(input('id', 0, 'intval'))){
            $this->error('请先安装此应用');
        }
        $mini_template = model('addonsTemplate')->getOneByMap(['addon' => $data['addon']], true, true);
        if(! $mini_template){
            $mini_template = ['addon' => $data['addon']];
        }
        $mini_template['addon_name'] = $data['name'];
        $templates = model('miniTemplate')->getFieldByOrder([
            'field' => 'template_id, user_desc',
            'where' => ['type' => \app\common\model\MiniTemplate::TEMPLATE],
            'order' => ['id' => 'desc'],
            'refresh' => true
        ]);
        $builder = new FormBuilder();
        $builder->addFormItem('addon', 'hidden', 'addon', 'addon')
            ->addFormItem('addon_name', 'static', '应用','应用')
            ->addFormItem('template_id', 'chosen', '选择模版', '选择模版', $templates, 'required')
            ->setFormData($mini_template);
        return $builder->show();
    }

    /**
     * 卸载应用
     * Author: fudaoji<fdj@kuryun.cn>
     */
    public function uninstall()
    {
        if (request()->isPost()) {
            $name = input('name', '');
            if ($name == '') {
                $this->error('没有要卸载的应用');
            }
            $path= ADDON_PATH . $name . DS;
            if(!file_exists($path)){
                $this->error($path.'目录不存在');
            }
            if(!is_writable($path)){
                $this->error($path.'目录没有删除权限');
            }

            controller('common/addon', 'event')->clearAddonData(['name' => $name]); //再次清空数据
            if(del_dir_recursively($path)){
                $this->success('应用卸载成功，数据及文件都已删除');
            }else{
                $this->error('删除应用目录失败');
            }
        }
    }

    /**
     * 应用还原
     * Author: fudaoji<fdj@kuryun.cn>
     */
    public function wipeData()
    {
        if (request()->isPost()) {
            $name = input('name', '');
            if ($name == '') {
                $this->error('没有要还原的应用');
            }
            $res = controller('common/addon', 'event')->clearAddonData(['name' => $name]);
            if($res === true){
                $this->success('应用还原成功');
            }else{
                $this->error('还原失败，失败原因请查看日志');
            }
        }
    }

    /**
     * 安装/启用应用扩展
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function install()
    {
        if (request()->isPost()) {
            $name = input('name', '');
            if ($name == '') {
                $this->error('name参数错误');
            }
            cache('callAddonCache'.$name, null);

            if ($addon = $this->addonM->getOneByMap(['addon' => $name], true, true)) {
                if ($addon['status'] == 1) {
                    $this->error('应用已安装，请先卸载再重新安装');
                } else {
                    //小程序应用必须关联了模版，才能上架
                    if($addon['type'] == AdminStore::MINI && ! model('addonsTemplate')->total(['addon' => $name], true)){
                        $this->error('请先为此应用设置模版', url('setTemplate', ['id' => $addon['id']]));
                    }
                    $this->addonM->updateOne(['id' => $addon['id'], 'status' => 1]);
                    $this->success('应用上架成功', url('index', ['type'=>'onsale']));
                }
            } else {
                $cf = $this->addonM->getAddonConfigByFile($name);
                $data = [
                    'name' => isset($cf['name']) ? $cf['name'] : '',
                    'addon' => isset($cf['addon']) ? $cf['addon'] : '',
                    'desc' => isset($cf['desc']) ? $cf['desc'] : '',
                    'version' => isset($cf['version']) ? $cf['version'] : '',
                    'author' => isset($cf['author']) ? $cf['author'] : '',
                    'logo' => isset($cf['logo']) ? ('/addons/' . $cf['addon'] . '/'.$cf['logo']) : '',
                    'menu_show' => isset($cf['menu_show']) ? $cf['menu_show'] : '1',
                    'entry_url' => isset($cf['entry_url']) ? $cf['entry_url'] : '',
                    'admin_url' => isset($cf['admin_url']) ? $cf['admin_url'] : '',
                    //'config' => isset($cf['config']) ? json_encode($cf['config']) : '',
                    'status' => 0,
                    'type' => isset($cf['type']) ? $cf['type'] : AdminStore::MP,
                ];

                $result = $this->validate($data, 'Addon');

                if ($result !== true) {
                    $this->error($result);
                }

                if (isset($cf['install_sql']) && $cf['install_sql'] != '') {
                    $install_file = ADDON_PATH . $name . DS . $cf['install_sql'];
                    if (!is_file($install_file)) {
                        $this->error('没有找到安装数据的SQL文件：' . $cf['install_sql']);
                    } else {
                        if (!strpos($install_file, '.sql')) {
                            $this->error('安装文件格式有误');
                        }
                        $res = execute_addon_install_sql($install_file);
                        if($res !== true){
                            $this->error($res);
                        }
                    }
                }
                if ($addon = $this->addonM->addOne($data)) {
                    $insert = ['id' => $addon['id']];
                    $remote_info = $this->fetchApp('groupcode');
                    if($remote_info !== false){
                        $insert['detail'] = empty($remote_info['data']['info']['detail']) ? '' : $remote_info['data']['info']['detail'];
                        $insert['cates'] = empty($remote_info['data']['info']['cates']) ? '' : $remote_info['data']['info']['cates'];
                        $insert['snapshot'] = empty($remote_info['data']['info']['snapshot']) ? '' : $remote_info['data']['info']['snapshot'];
                    }
                    $this->addonInfoM->addOne($insert);
                    $extra_msg = $data['type'] == AdminStore::MINI ? ',您需要为此应用设置模版才可以正式上架' : '';
                    $this->success('安装应用成功' . $extra_msg, url('index', ['type'=>'notinstall']));
                }else{
                    $this->error('安装应用失败');
                }
            }
        }
    }

    /**
     * 下架
     * @throws \think\Exception
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     * Author: fudaoji<fdj@kuryun.cn>
     */
    public function close()
    {
        if (request()->isPost()) {
            $name = input('name', '');
            $addon = $this->addonM->getOneByMap(['addon' => $name, 'status' => 1], true, true);
            if (empty($addon)) {
                $this->error('没有要停用的应用');
            }
            cache('callAddonCache'.$name, null);

            if ($this->addonM->updateOne(['status' => 0, 'id' => $addon['id']])) {
                $this->success('停用应用成功');
            }
            $this->error('停用应用失败');
        }
    }


    /**
     * 应用中心
     * @param string $type
     * @return mixed
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function index($type = 'uninstall')
    {
        $search_key = input('search_key', '');
        $app_type = input('app_type', 'all');
        $where = ['a.status' => 1];
        $type == 'onsale' && $where['a.status'] = 0;
        $search_key && $where['name'] = ['like', '%'.$search_key.'%'];
        $app_type !== 'all' && $where['type'] = ['like', '%'.$app_type.'%'];
        $apps = $this->addonM->pageJoin([
            'alias' => 'a',
            'join' => [
                ['addons_info ai', 'ai.id=a.id'],
                ['addons_template at', 'at.addon=a.addon', 'left']
            ],
            'page_size' => $this->pageSize,
            'where' => $where,
            'order' => ['ai.sale_num' => 'desc'],
            'field' => 'a.*,ai.sale_num,ai.price,at.template_id',
            'refresh' => 1
        ]);
        $page = $apps->appends([
            'search_key' => $search_key,
            'app_type' => $app_type,
            'type' => $type])->render();

        $addon_list = $apps;

        if ($type == 'notinstall') { //未安装
            $addon_folder = opendir(ADDON_PATH);
            $addons = []; //存放未安装的插件
            if ($addon_folder) {
                while (($file = readdir($addon_folder)) !== false) {
                    if ($file != '.' && $file != '..') {
                        if ($addon_local = $this->addonM->getAddonConfigByFile($file)) {
                            $addon_db = $this->addonM->getOneByMap(['addon' => $file], true, 1);
                            if (empty($addon_db)) {
                                $addons[] = $addon_local;
                            }
                        }
                    }
                }
            }
            $addon_list = $addons;
        }

        foreach ($addon_list as $key => $value) {
            $temp = $addon_list[$key];
            $temp['type'] = explode(',', $value['type']);
            if($type == 'notinstall'){
                $temp['logo'] = '/addons/'.$value['addon'].'/' . $value['logo'];
            }
            $addon_list[$key] = $temp;
        }

        $assign = [
            'addons' => $addon_list,
            'apps' => $apps,
            'type' => $type,
            'app_type' => $app_type,
            'search_key' => $search_key,
            'page' => $page
        ];
        return $this->show($assign);
    }

    /**
     * 编辑应用信息
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function edit(){
        if(request()->isPost()){
            $post_data = input('post.');
            unset($post_data['__token__']);
            $this->addonM->updateOne(['id' => $post_data['id'], 'version' => $post_data['version']]);
            unset($post_data['version']);
            $this->addonInfoM->updateOne($post_data);
            $this->success('保存成功', url('index'));
        }
        if(! $data = $this->addonM->getOne(input('id', 0, 'intval'))){
            $this->error('数据不存在');
        }

        $data = array_merge($data, $this->addonInfoM->getOne($data['id']));
        $data['cates'] = empty($data['cates']) ? [] : explode(',', $data['cates']);

        $cates = array_values(model('addonsCate')->getField('title', ['status' => 1], true));

        $builder = new FormBuilder();
        $builder->addFormItem('id', 'hidden', 'id', 'id')
            ->addFormItem('cates', 'chosen_multi', '分类标签', '可多选', array_combine($cates, $cates), 'required')
            ->addFormItem('version', 'text', '版本', '版本', [], 'required')
            ->addFormItem('sale_num_show', 'number', '虚拟销量', '前台显示的数字', [], 'required min=0')
            ->addFormItem('old_price', 'text', '原价', '原价', [], 'required min="0"')
            ->addFormItem('price', 'text', '售价', '每年的费用', [], 'required min="0"')
            ->addFormItem('snapshot', 'pictures_url', '应用快照', '应用典型界面截图', [], 'required')
            ->addFormItem('detail', 'ueditor', '详细介绍', '详细介绍', [], 'required max=50000')
            ->setFormData($data);

        return $builder->show();
    }

    /**
     * 从官方市场获取应用详情
     * @param int $name
     * @return mixed
     * Author: fudaoji<fdj@kuryun.cn>
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    protected function fetchApp($name = ''){
        $request = new \GuzzleHttp\Psr7\Request('post', 'app/getByNamePost');
        $client = new \GuzzleHttp\Client(['base_uri' => 'https://kyphp.kuryun.com/api/']);
        try {
            $res = $client->send($request, [
                'json' => ['name' => $name]
            ]);
            if($res->getStatusCode() == 200){
                return json_decode($res->getBody()->getContents(), true);
            }
        }catch (\Exception $e){
            Log::error($e->getMessage());
        }
        return false;
    }
}